using System;
using System.CodeDom;
using System.IO;
using System.Web.Compilation;
using System.Xml;

namespace MyTestSite
{
	public class MyBuildProvider : BuildProvider
	{
		public override void GenerateCode(AssemblyBuilder assemblyBuilder)
		{
			CodeCompileUnit codeUnit = new CodeCompileUnit();
			XmlDocument xDoc = OpenXmlDoc();
			foreach(XmlElement nsElement in xDoc.SelectNodes("//namespace"))
			{
				codeUnit.Namespaces.Add(ToNamespace(nsElement));
			}
			assemblyBuilder.AddCodeCompileUnit(this, codeUnit);
		}

		private CodeNamespace ToNamespace(XmlElement nsElement)
		{
			CodeNamespace ns = new CodeNamespace(nsElement.Attributes["name"].Value);
			foreach (XmlElement classElement in nsElement.SelectNodes("//class"))
			{
				ns.Types.Add(ToClass(classElement));
			}

			return ns;
		}

		private CodeTypeDeclaration ToClass(XmlElement classElement)
		{
			CodeTypeDeclaration type = new CodeTypeDeclaration(classElement.Attributes["name"].Value);
			foreach (XmlElement propElement in classElement.SelectNodes("field"))
			{
				type.Members.Add(ToField(propElement));
			}

			return type;
		}

		private CodeMemberField ToField(XmlElement fieldElement)
		{
			CodeMemberField field = new  CodeMemberField(typeof(string), fieldElement.Attributes["name"].Value);
			field.Attributes = MemberAttributes.Const | MemberAttributes.Public;
			field.InitExpression = new CodePrimitiveExpression(fieldElement.Attributes["value"].Value);

			return field;
		}

		private XmlDocument OpenXmlDoc()
		{
			XmlDocument xDoc = new XmlDocument();
			using (TextReader reader = this.OpenReader())
			{
				xDoc.Load(reader);
			}

			return xDoc;
		}
	}
}